import base64
import os
import sqlite3

static_images = os.path.dirname(os.path.abspath(__file__)) + "/static/images/styles/small/"
# static_images = os.path.dirname(os.path.abspath(__file__)) + "/static/images/styles/small/"
save_parent_folder = "/storage/emulated/0/Music/EisenRadio"
script_path = os.path.dirname(__file__)


def convert_ascii(file_name):
    with open(file_name, "rb") as reader:
        img_bytes = reader.read()
        img_ascii = render_picture(img_bytes, 'encode')
    return img_ascii


def render_picture(byte_data, de_enc):
    render_pic = ''
    if de_enc == 'encode':
        render_pic = base64.b64encode(byte_data).decode('ascii')
    if de_enc == 'decode':
        render_pic = base64.b64decode(byte_data).decode('ascii')
    return render_pic


def make_db_from_schema(db_path):
    # # # # custom filled db # # #

    # connection = sqlite3.connect((os.path.join(script_path, 'database.db')))
    connection = sqlite3.connect(str(db_path))
    with open((os.path.join(script_path, 'schema.sql'))) as f:
        connection.executescript(f.read())

    cur = connection.cursor()
    cur.execute("INSERT INTO eisen_intern (browser_open,statistics,commercials) VALUES (?,?,?)", (str(1), 1, 1))

    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('classic',
         'http://37.251.146.169:8000/streamHD',
         save_parent_folder,
         'Stream HE-AAC @ 192 kbps, 44.1 kHz\n'
         '\n'
         'Listener Peak (limit):	200\n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-brown_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('PARTY-VIBE-RADIO',
         'http://94.130.242.5:8010/stream',
         save_parent_folder,
         'https://www.partyvibe.com/ \n'
         '\n'
         'multiple streams at \n'
         'http://94.130.242.5:8010 \n'
         '\n'
         'Radio station currently plays host to 10 channels of\n'
         'streaming audio covering the following musical styles:\n'
         '\n'
         'ambient, breakbeat, drum & bass, dubstep, \n'
         'pop, psychedelic trance, rap, reggae, rock and techno music.\n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-black_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('audio_noir',
         'http://104.192.169.54:8000/noir',
         save_parent_folder,
         'usa - Classic Old Time Radio, Sci Fi, Comedy, Drama \n'
         ' \n'
         'Use the URL from header to go to their page. \n'
         'They offer a download site and preview for the next 10 upcoming tracks. \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-blue_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('lounge',
         'http://streamingv2.shoutcast.com/japanimradio-tokyo',
         save_parent_folder,
         'They send no header info for their Web page. \n'
         ' \n'
         'Search engine "Jamendo lounge" helps. \n'
         'Free tracks and playlists. \n',
         "image/jpeg",
         convert_ascii(static_images + "mixer-tempelhof-airport.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('radio_pan_america',
         'http://51.222.8.101:8000/stream',
         save_parent_folder,
         'Radio Panamericana, Lima - Peru \n'
         'Salsa \n'
         ' \n'
         'Big Web site. \n'
         'You can use "Edit" database to go to their Web site. \n'
         ' \n'
         'They have a user limit. \n',
         "image/jpeg",
         convert_ascii(static_images + "mixer-reload.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('YeahMon',
         'http://c3.radioboss.fm:8095/autodj',
         save_parent_folder,
         'Yeah Mon Radio \n'
         'Reggae, Soca, Afrobeats, RnB, Hip Hop, Pop \n'
         ' \n'
         'yeahmonradio@gmail.com \n'
         'Tel 678-778-3401 \n'
         ' \n'
         'Republic of Vanuatu, island country in Melanesia,\n'
         'located in the South Pacific Ocean.\n',
         "image/jpeg",
         convert_ascii(static_images + "mixer-construction-site.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('playUrban',
         'http://live.playradio.org:9090/UrbanHD',
         save_parent_folder,
         'Play urban, Romania \n'
         ' \n'
         'You can use "Edit" database to go to their Web site. \n'
         'They have multiple streams and a you can request a song. \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-red_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Nachtflug',
         'http://85.195.88.149:11810/sid=1',
         save_parent_folder,
         'Nachtflug, Germany \n'
         'Goth, Industrial, Electronic \n'
         ' \n'
         'They have a broadcast schedule and you can request a song. \n'
         'You can use "Edit" database to go to their Web site. \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-violet_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('hm',
         'https://hirschmilch.de:7001/prog-house.mp3',
         save_parent_folder,
         'Hirschmilch Radio - hirschmilch.de \n'
         ' \n'
         'Radio Shows.'
         ' \n'
         'Hirschmilch Radio Channels \n'
         'Chillout, Electronic, Prog-House, Progressive, Psytrance, Techno \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-whiteneongelb_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('zenStyle',
         'https://radio4.cdm-radio.com:18004/stream-mp3-Zen',
         save_parent_folder,
         'Radio Costa Del Mar \n'
         ' \n'
         'Channels: \n'
         'CHILLOUT, DEEP-HOUSE, SMOOTH, JAZZ, DANCE, FUNKY, ZEN \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-white_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Paloma',
         'https://pool.radiopaloma.de/RADIOPALOMA.mp3',
         save_parent_folder,
         'Berlin /Germany \n'
         'Schlager \n'
         ' \n'
         'You can use "Edit" database to go to their Web site. \n'
         'Some advertising. \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-brown_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Bayern1',
         'https://streams.br.de/bayern1obb_2.m3u',
         save_parent_folder,
         'Bayern Germany \n'
         'Pop \n'
         ' \n'
         'An example for a m3u playlist server. It sends a list of streams. \n'
         'EisenRadio will connect to the first listed radio station. \n'
         'https://streams.br.de/bayern1obb_2.m3u \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-black_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Reggae',
         'http://hd.lagrosseradio.info:8000/lagrosseradio-reggae-192.mp3',
         save_parent_folder,
         'Paris / France \n'
         'REGGAE \n'
         ' \n'
         '3 webradios et 3 webzines gratuits, militants et 100% indépendants \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-neongreen_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('aacChill',
         'http://radio4.vip-radios.fm:8020/stream128k-AAC-Chill_autodj',
         save_parent_folder,
         'Radio Costa Del Mar \n'
         'chillout \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-neongreen_120x200.png"))
        )
    connection.commit()
    connection.close()


def empty_db_from_schema():
    # # # # custom filled db # # #
    connection = ''
    is_snap_device = 'SNAP' in os.environ
    is_android_device = 'ANDROID_STORAGE' in os.environ

    if is_android_device:
        return
    if not is_snap_device:
        connection = sqlite3.connect((str(os.path.join(script_path, 'database.db'))))
    if is_snap_device:
        connection = sqlite3.connect((str(os.path.join(os.environ["SNAP_USER_COMMON"], 'pre_configured.db'))))

    with open((os.path.join(script_path, 'schema.sql'))) as f:
        connection.executescript(f.read())

    cur = connection.cursor()
    cur.execute("INSERT INTO eisen_intern (browser_open,statistics,commercials) VALUES (?,?,?)", (str(1), 1, 1))
    connection.commit()
    connection.close()



import os
from flask import flash
from eisenradio.api import eisenApi


def make_folder(this_folder):
    rv = True
    try:
        os.makedirs(this_folder)
        flash('Done!\t' + this_folder, 'success')
    except FileExistsError:
        pass
    except Exception as e:
        flash(f'Failed!\t {e}', 'danger')
        rv = False
    finally:
        return rv


def is_in_db_view(a_name):
    for radio_name in eisenApi.radio_id_name_dict.values():
        if radio_name.lower() == a_name.lower():
            return True
    return False


def remove_blank(a_name):
    rv = a_name.replace(" ", "")
    return rv


def remove_special_chars(str_name):
    str_name.replace(" ", "")
    # my_str = "hey th~!ere. /\ coolleagues?! Straße"
    rv = str_name.translate({ord(string): "" for string in '"!@#$%^*()[]{};:,./<>?\\|`~=+"""'})
    return rv


def radio_start_page_url_get(radio_name):
    """return the anchor point url with the port number of the server"""
    radio_id = ""
    for id_num, name in eisenApi.radio_id_name_dict.items():
        if name == radio_name:
            radio_id = id_num
    port = eisenApi.work_port
    radio_start_page_url = 'http://localhost:' + str(port) + '/#dot_' + str(radio_id)
    return radio_start_page_url


def radio_listen_button_get():
    """ return complete id of current pressed listen button"""
    l_list = [radio_id for radio_id, btn_down in eisenApi.lis_btn_dict.items() if btn_down]
    button_id = "Listen_" + str(l_list[0]) if len(l_list) > 0 else ""
    return button_id


def radio_name_get():
    """return name if id is in list else None"""
    l_list = [radio_id for radio_id, btn_down in eisenApi.lis_btn_dict.items() if btn_down]
    if len(l_list) > 0:
        radio_id = l_list[0]
        return eisenApi.radio_id_name_dict[radio_id]
    return


def parse_url_simple_url(radio_url):
    """called by routes.py, def post(post_id) to link to original radio URL """
    url = radio_url  # whole url is used for connection to radio server

    # 'http:' is first [0], 'ip:port/xxx/yyy' second item [1] in list_url_protocol
    list_url_protocol = url.split("//")
    list_url_ip_port = list_url_protocol[1].split("/")  # 'ip:port' is first item in list_url_ip_port
    radio_simple_url = list_url_protocol[0] + '//' + list_url_ip_port[0]
    return radio_simple_url



from ghettorecorder import ghettoApi
from eisenradio.api import eisenApi


def header_data_read(name):
    """ Return header info data for the arg id.
    """
    return request_header_info(name)


def request_header_info(name):
    """ Return a list with collected response header strings if available. """
    info_dct = {
        "request_time": request_time_api(name),
        "request_suffix": request_suffix_api(name),
        "request_icy_genre": request_icy_genre_api(name),
        "request_icy_name": request_icy_name_api(name),
        "request_icy_view": request_icy_view_id_api(name),
        "request_icy_br": request_icy_br_api(name),
        "request_icy_url": request_icy_url_api(name),
        "current_song": request_current_song_api(name),
    }
    return info_dct


def request_suffix_api(name):
    """ Return the content type resolved to file suffix string. """
    rv = '---'
    try:
        content_type = ghettoApi.ghetto_measure_dict[name + ',suffix']
        if content_type == 'audio/aacp' or content_type == 'application/aacp':
            rv = 'aacp'
        if content_type == 'audio/aac':
            rv = 'aac'
        if content_type == 'audio/ogg' or content_type == 'application/ogg':
            rv = 'ogg'
        if content_type == 'audio/mpeg':
            rv = 'mp3'
        if content_type == 'audio/x-mpegurl' or content_type == 'text/html':
            rv = 'm3u'
    except KeyError:
        pass
    return str(rv)


def request_icy_view_id_api(name):
    rv = '---'
    try:
        for key, val in eisenApi.radio_id_name_dict.items():
            if val == name:
                rv = key
    except KeyError:
        pass
    return str(rv)


def request_time_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',request_time']
    except KeyError:
        pass
    return str(rv)


def request_icy_genre_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_genre']
    except KeyError:
        pass
    return str(rv)


def request_icy_name_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_name']
    except KeyError:
        pass
    return str(rv)


def request_icy_br_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_br']
    except KeyError:
        pass
    return str(rv)


def request_icy_url_api(name):
    rv = '---'
    try:
        rv = ghettoApi.ghetto_measure_dict[name + ',icy_url']
    except KeyError:
        pass
    return str(rv)


def request_current_song_api(name):
    rv = '---'
    try:
        rv = ghettoApi.current_song_dict[name]
    except KeyError:
        pass
    return str(rv)



""" Flask application factory with blueprints
'Home' and 'Util' load modules, templates, styles, favicon from its own project folders
"""
import certifi
from flask import Flask
from os import path, environ
from eisenradio.api import api, eisenApi

# android ssl fix
environ['SSL_CERT_FILE'] = certifi.where()

script_path = path.dirname(__file__)


def create_app(work_port):
    """ flask prod application factory
    prod
    """
    app = Flask('eisenradio')

    with app.app_context():

        api.init_app(app)
        eisenApi.init_work_port(work_port)  # port to use; browser autostart, sound endpoint

        is_snap_device = 'SNAP' in environ  # write in [SNAP_USER_COMMON]
        is_android_device = 'ANDROID_STORAGE' in environ

        if not is_snap_device and not is_android_device:
            # PROD
            app.config.from_object('config.ProdConfig')
            # app.config.from_object('config.TestConfig')    # total fail from pytest

        if is_snap_device:
            # write_config('snap')
            # remove_config(environ["SNAP_USER_COMMON"])
            app.config.from_object('config.SnapConfig')
            pass

        if is_android_device:
            app.config.from_object('config.AndroidConfig')

        # helper stuff
        from eisenradio.lib.platform_helper import main as start_frontend
        from eisenradio.lib.eisdb import install_new_db as create_install_db

        from eisenradio.eisenhome import routes as home_routes
        from eisenradio.eisenutil import routes as util_routes

        # Register Blueprints (pointer to parts of the application, subprojects in production)
        app.register_blueprint(home_routes.eisenhome_bp)
        app.register_blueprint(util_routes.eisenutil_bp)

        create_install_db(app.config['DATABASE'])

        start_frontend()
        return app


def create_app_test(work_port):
    """test
    hours of senseless tries to start test by ... app.config.from_object('config.TestConfig')
    use .env loader to load config in OS vars, then app.config.update() from those vars
    test
    """
    app = Flask('eisenradio')

    with app.app_context():

        api.init_app(app)
        eisenApi.init_work_port(work_port)  # port to use; browser autostart, sound endpoint

        # helper stuff
        from eisenradio.lib.platform_helper import main as start_frontend
        from eisenradio.lib.eisdb import install_new_db as create_install_db

        from eisenradio.eisenhome import routes as home_routes
        from eisenradio.eisenutil import routes as util_routes

        # Register Blueprints (pointer to parts of the application, subprojects in production)
        app.register_blueprint(home_routes.eisenhome_bp)
        app.register_blueprint(util_routes.eisenutil_bp)

        app.config.update(
            SECRET_KEY=environ['SECRET_KEY'],
            DATABASE=environ['DATABASE'],
            FLASK_ENV=environ['FLASK_ENV'],
            DEBUG=environ['DEBUG'],
            TESTING=environ['TESTING'],
            PYTEST=True
        )

        create_install_db(app.config['DATABASE'])

        start_frontend()
        return app



import os
from flask import Blueprint, render_template, request, url_for, flash, redirect, make_response, jsonify, Response
from aacrepair import AacRepair
from ghettorecorder import ghettoApi
import eisenradio.eisenutil.eisutil as eis_util
from eisenradio.eisenutil import config_html
from eisenradio.eisenutil import request_info
from eisenradio.eisenutil import tools as util_tools
from eisenradio.eisenutil import monitor_records as mon_rec
import eisenradio.eisenutil.browser_stream as browser_stream
import eisenradio.eisenutil.stopped_stations as stopped_stations
from eisenradio.lib import eisdb as lib_eisdb
from eisenradio.api import api, eisenApi


blacklist_enable = False
ghettoApi.init_blacklist_enable(blacklist_enable)
aac_repair_log = []
ghettoApi.init_aac_repair_log(aac_repair_log)

# Blueprint Configuration
eisenutil_bp = Blueprint(
    'eisenutil_bp', __name__,
    template_folder='bp_util_templates',
    static_folder='bp_util_static',
    static_url_path='/bp_util_static'
)


@eisenutil_bp.route('/tools_transparent_image_load', methods=['POST'])
def tools_transparent_image_load():
    """ return list to JS div list maker, replace standard random pic with a translucent one """
    request_dict = request.form.to_dict()
    radio_name = request_dict['radioName']
    eisenApi.init_radio_name_id_dict()   # need db request if new radio was assigned or new in .radio_name_id_dict
    util_tools.radio_transparent_image_db(eisenApi.radio_name_id_dict[radio_name])
    msg_list = ["Radio got a beautiful transparent image."]
    return jsonify({"transparentImageLoad": msg_list})


@eisenutil_bp.route('/tools_radio_html_settings', methods=['POST'])
def tools_radio_html_settings():
    """ update front-end settings, css style and animations in database
    send a message log to JS

    show_html_animation:
       show/hide all animations

    show_html_style:
       show/hide css style

    show_html_front_pigs:
       not actually pigs, but penguin, bear and tiger

    show_html_ballon:
        show/hide all aircraft related animations, balloon, parachute drop

    show_html_speaker:
       remove speaker animation

    return:
       list of messages for each config item to allow easy loop in js with list made from divs
    """
    on = 1
    off = 0
    show_html_animation = 1
    show_html_style = 1
    show_html_front_pigs = 1
    show_html_ballon = 1
    show_html_speaker = 1

    row_db_html_animation = 3
    row_db_html_style = 4
    row_db_html_front_pigs = 5
    row_db_html_balloon = 6
    row_db_html_speaker = 7

    request_dict = request.form.to_dict()
    config_animation = True
    msg_no_style_needed = ""
    for key, value in request_dict.items():
        if key == "checkboxConfigAnimation" and value == "true":
            show_html_animation = config_html.tools_feature_toggle_show_html_config(on, row_db_html_animation)
        if key == "checkboxConfigAnimation" and value == "false":
            show_html_animation = config_html.tools_feature_toggle_show_html_config(off, row_db_html_animation)
            config_animation = False
        if key == "checkboxConfigStyle" and value == "true":
            # if no animation is selected the style is senseless
            if not config_animation:
                msg_no_style_needed = "--> no animation at all, deactivate style too"
                show_html_style = config_html.tools_feature_toggle_show_html_config(off, row_db_html_style)
            else:
                show_html_style = config_html.tools_feature_toggle_show_html_config(on, row_db_html_style)
        if key == "checkboxConfigStyle" and value == "false":
            show_html_style = config_html.tools_feature_toggle_show_html_config(off, row_db_html_style)
        if key == "checkboxConfigFrontPigs" and value == "true":
            show_html_front_pigs = config_html.tools_feature_toggle_show_html_config(on, row_db_html_front_pigs)
        if key == "checkboxConfigFrontPigs" and value == "false":
            show_html_front_pigs = config_html.tools_feature_toggle_show_html_config(off, row_db_html_front_pigs)
        if key == "checkboxConfigBalloon" and value == "true":
            show_html_ballon = config_html.tools_feature_toggle_show_html_config(on, row_db_html_balloon)
        if key == "checkboxConfigBalloon" and value == "false":
            show_html_ballon = config_html.tools_feature_toggle_show_html_config(off, row_db_html_balloon)
        if key == "checkboxConfigFlatSpeaker" and value == "true":
            show_html_speaker = config_html.tools_feature_toggle_show_html_config(on, row_db_html_speaker)
        if key == "checkboxConfigFlatSpeaker" and value == "false":
            show_html_speaker = config_html.tools_feature_toggle_show_html_config(off, row_db_html_speaker)

    # msg implemented as fake list, each list item gets a div to make coloring and font styling easy, if wanted
    msg_animation_value = "enabled" if show_html_animation else "disabled"
    msg_style_value = "enabled" if show_html_style else "disabled"
    msg_animals_value = "enabled" if show_html_front_pigs else "disabled"
    msg_balloon_value = "enabled" if show_html_ballon else "disabled"
    msg_speaker_value = "enabled" if show_html_speaker else "disabled"

    msg = [".", "Hello, here your HTML settings:", ".",
           f"Style {msg_style_value}",
           f"Animation {msg_animation_value}",
           f"inflated animals {msg_animals_value}",
           f"animated balloon {msg_balloon_value}",
           f"animated speaker {msg_speaker_value}",
           msg_no_style_needed,
           ]
    return jsonify({"configEisenradioUpdate": msg})


@eisenutil_bp.route('/tools_aacp_repair_log', methods=['GET'])
def tools_aacp_repair_log():
    """ return log to JS, where related lines are colored """
    repair_log = ghettoApi.aac_repair_log
    ghettoApi.aac_repair_log = []
    return jsonify({"aacRepairPullLog": repair_log})


@eisenutil_bp.route('/tools_aacp_repair', methods=['POST'])
def tools_aacp_repair():
    """return flash warning if no aac file else create dict with file name and bytes content, redirect to start page

    sort out non aac files, call tool_aacp_repair(), threaded, so can not use flash to show the numbers, use log file
    flash the path to the written log file
    write acp dict with files to work on, convert file.storage object f.read() to bytes {file(n).aac: bytes(content)}
    """
    files = request.files.getlist('fileUploadAcpRepair')

    name_list = [f.filename for f in files if f.filename[-5:] == ".aacp" or f.filename[-4:] == ".aac"]
    if len(name_list) == 0:
        flash("No aac files.", 'warning')
        return render_template('bp_util_aac_repair.html')

    f_dict = {f.filename: f.read() if f.filename[-5:] == ".aacp" or f.filename[-4:] == ".aac" else None for f in files}
    export_path = lib_eisdb.get_download_dir()
    aac_rep = AacRepair(export_path, f_dict)
    aac_rep.repair()
    ghettoApi.aac_repair_log.clear()
    ghettoApi.aac_repair_log = aac_rep.log_list
    return render_template('bp_util_aac_repair.html')


@eisenutil_bp.route('/tools_radio_blacklist_set', methods=['GET'])
def tools_radio_blacklist_set():
    """enable blacklist first time, then switch status, return flash enabled/disabled, redirect to start page

    a call to this endpoint will change the value in db and delete all lists of all radios in db
    button is fashioned in tools()
    """
    is_enabled = mon_rec.status_db_blacklist_get()
    enabled = mon_rec.feature_blacklist_switch_status(is_enabled)
    mon_rec.delete_all_blacklists(enabled)
    rv = {"enabled": True} if enabled else {"enabled": False}
    return rv


@eisenutil_bp.route('/<string:radio_name>/tools_blacklist_skipped', methods=['GET'])
def tools_blacklist_skipped(radio_name):
    """html page show the count of skipped titles per session and a list with their names

    add link to return to radio on start page
    vars
    radio_start_page_url -  to return directly to correct formatted page with radio on top of page
    radio_name
    skiplist - user info, list of title names skipped during session, can contain many duplicates if radio is info radio
    skip_count - user info how often titles of this radio were skipped during session because of using blacklist feature
    """
    radio_start_page_url = eis_util.radio_start_page_url_get(radio_name)

    skiplist, skip_count = '', 0
    try:
        skiplist = ghettoApi.skipped_in_session_dict[radio_name]
        skip_count = len(skiplist)
    except KeyError:
        pass
    return render_template('bp_util_skip_blacklist.html',
                           radio_start_page_url=radio_start_page_url,
                           radio_name=radio_name,
                           skiplist=skiplist,
                           skip_count=skip_count)


@eisenutil_bp.route('/tools_blacklist_overview', methods=['GET'])
def tools_blacklist_overview():
    """creates a html page with buttons for every radio to go to the blacklists edit page, returns vars for page build

    vars
    radios_dict - all radios loaded if start page is drawn ghettoApi.radio_id_name_dict
    streamer_name_list - change btn color for radios with active rec
    skip_count - user info how often titles of this radio were skipped during session because of using blacklist feature
    radio_blacklist_count = shows the count of blacklisted titles
    """

    skip_count = 0
    view_dict = eisenApi.radio_id_name_dict  # key db id: val name
    skip_title_dict = ghettoApi.skipped_in_session_dict  # key radio: val title list
    blacklist_dict = ghettoApi.all_blacklists_dict
    streamer_name_list = []
    radio_blacklist_count = {}

    for db_id, btn_pressed in eisenApi.rec_btn_dict.items():
        if btn_pressed:
            try:
                streamer_name_list.append(view_dict[db_id])
            except KeyError:
                pass
    for title_list in skip_title_dict.values():
        if title_list is not None:
            for _ in title_list:
                skip_count += 1
    for radio, title in blacklist_dict.items():
        blacklist_count = 0
        if title is not None:
            for _ in title:
                blacklist_count += 1
            radio_blacklist_count[radio] = blacklist_count

    return render_template('bp_util_radio_all_blacklist.html',
                           radios_dict=view_dict,
                           streamer_name_list=streamer_name_list,
                           skip_count=skip_count,
                           radio_blacklist_count=radio_blacklist_count)


@eisenutil_bp.route('/<string:radio_name>/tools_radio_blacklist', methods=['GET'])
def tools_radio_blacklist(radio_name):
    """on click render html page for a blacklist of one radio, returns vars to build the list

    vars
    radio_start_page_url - to return directly to correct formatted page with radio on top of page
    len(blacklist) - write number of blacklist items in the list beneath the radio name
    sorted_title_dict - to show titles asc. sorted and remember index nums of org list for title deletion in org list
    skip_count - user info how often titles of this radio were skipped during session because of using blacklist feature
    radio_name
    """
    radio_start_page_url = eis_util.radio_start_page_url_get(radio_name)

    blacklist = mon_rec.dump_radio_blacklist_from_col(radio_name)
    sorted_title_dict = mon_rec.sort_dictionary_by_value(blacklist)
    skip_count = 0
    try:
        skip_count = len(ghettoApi.skipped_in_session_dict[radio_name])
    except KeyError:
        pass
    return render_template('bp_util_radio_blacklist.html',
                           radio_start_page_url=radio_start_page_url,
                           counter=len(blacklist),
                           sorted_title_dict=sorted_title_dict,
                           skip_count=skip_count,
                           radio_name=radio_name)


@eisenutil_bp.route('/tools_radio_blacklist_del_from_list', methods=['POST'])
def tools_radio_blacklist_del_from_list():
    """from html blacklist page delete one or more title from the list, or delete the whole list

    click java: delFromBlacklist(), html: bp_util_radio_blacklist.html
    """
    log = "-empty-"
    if request.method == 'POST':
        request_dict = request.form.to_dict()
        radio_name = request_dict['radio_name']
        delete_list = request_dict['delAll']
        # java
        if delete_list == 'false':
            log = mon_rec.del_single_title_master(radio_name, request_dict)
        if delete_list == 'true':
            log = mon_rec.delete_blacklist(radio_name)
        msg = "_ok_"
        return jsonify({'delFromBlacklist': msg,
                        'logFileDeleteFromBlackList': log})


@eisenutil_bp.route('/tools_export_blacklists', methods=['POST'])
def tools_export_blacklists():
    """call dump_radio_blacklist() export blacklists and return flash message if ok or not

    confirm with flask flash template, java popup kills selenium tests
    """
    rv = util_tools.dump_radio_blacklist()
    if rv:
        flash('Successfully exported!', 'success')
        return render_template('bp_util_flash.html')
    else:
        flash('Something went wrong, write protected?', 'warning')
        return render_template('bp_util_flash.html')


@eisenutil_bp.route('/tools_upload_blacklists', methods=['POST'])
def tools_upload_blacklists():
    """ call upload_blacklists to upload blacklists to server db

    Return
       flash message if ok or not, except on error, restore the blacklists from json file
    """
    file = request.files['fileUploadBlackLists']
    try:
        json_file = file.read()
        rv, count = util_tools.upload_blacklists(json_file)
    except Exception as error:
        print(error)
        return f'Something went wrong. error :: {error}'
    if rv:
        status = "Monitor Records is ON" if ghettoApi.blacklist_enable else "Monitor Records is OFF"
        flash(f'{count} unwanted titles imported! Lists are loaded. {status}', 'success')
        return render_template('bp_util_flash.html')
    else:
        flash('Something went wrong, use "Tools" Menu to go back! Wrong text utf-format? Wrong file? \nRead terminal '
              'messages, please. If possible', 'warning')
        return render_template('bp_util_flash.html')


@eisenutil_bp.route('/blacklist_enabled_get', methods=['GET'])
def blacklist_enabled_get():
    """ Return status from DB row object. Blacklist enabled 0, 1.
    Animate button or text.
    """
    blacklist_is_enabled = mon_rec.status_db_blacklist_get()
    return jsonify({'blacklistEnableGet': blacklist_is_enabled[0]})


@eisenutil_bp.route('/tools_delete_all', methods=['GET'])
def tools_delete_all():
    """delete all radios from db, return flash if ok or not

    used to restore from ini file with preferred radios and urls afterward
    """
    rv = util_tools.delete_all_radios()
    is_deleted = {"removed": True} if rv else {"removed": False}
    return is_deleted


@eisenutil_bp.route('/tools_database_delete', methods=['GET'])
def tools_database_delete():
    """return *ok*
    """
    is_deleted = {"removed": True}
    try:
        os.remove(api.config['DATABASE'])
        print("% s removed successfully" % api.config['DATABASE'])
    except OSError as error:
        print(error)
        print("% s error file not found." % api.config['DATABASE'])
        is_deleted = {"removed": False}

    print("foo")
    return is_deleted


@eisenutil_bp.route('/tools_export_ini', methods=['POST'])
def tools_export_ini():
    """call export_radios() to dump radio urls to ini file, return flash if ok or not"""
    rv = util_tools.export_radios()
    if rv:
        flash('Successfully exported!', 'success')
        return render_template('bp_util_flash.html')
    else:
        flash('Something went wrong, write protected?', 'warning')
        return render_template('bp_util_flash.html')


@eisenutil_bp.route('/tools_upload_ini', methods=['POST'])
def tools_upload_ini():
    """call upload_radios() to restore radio in db, return a flash message if ok or not"""
    file = request.files['fileUploadUrls']
    print(f' name {file.name} content-type {file.content_type}')
    try:
        settings_ini = file.read()
        rv, counter = util_tools.upload_radios(settings_ini.decode('ascii'))
    except Exception as error:
        print(error)
        return f'Something went wrong. error :: {error}'
    if rv:
        num_radios_added = "No new names imported!" if not counter else f"{counter} radios imported! Restart the App!"
        flash(f' {num_radios_added}', 'success')
        return render_template('bp_util_flash.html')
    else:
        flash('Something went wrong, use "Tools" Menu to go back! Wrong text utf-format? Wrong file? \nRead terminal '
              'messages, please. If possible', 'warning')
        return render_template('bp_util_flash.html')


@eisenutil_bp.route('/tools', methods=('GET', 'POST'))
def tools():
    """ render TOOLS page

    Info
       html config status switch checkboxes
       button to switch blacklist feature on/off
       blacklist_check_box_off is the digit part of a deselected checkbox unicode symbol
       in html hardcoded is & and # since I can not transfer those characters (don't know why)
       display path for exported files (same as parent download path)

    vars
       export_path - parent directory of radio folders
       button_label - blacklist feature button text
       button_message - blacklist feature info
       enabled - blacklist feature on or off
    """
    html_animation_row = 3
    html_style_row = 4
    html_front_pigs_row = 5
    html_balloon_row = 6
    html_speaker_row = 7

    anim_on = config_html.tools_feature_on_off_state_in_db(html_animation_row)
    animation_check_box_checked = 'checked' if anim_on else ' '
    animation_check_box_msg = "Stop all animations." if anim_on else "Enable animations. (required)"

    front_pigs_on = config_html.tools_feature_on_off_state_in_db(html_front_pigs_row)
    front_pigs_check_box_checked = 'checked' if front_pigs_on else ' '
    fp_msg = "Deselect a black bird show." if front_pigs_on else "Check out an exiting bird show."
    front_pigs_check_box_msg = fp_msg

    balloon_on = config_html.tools_feature_on_off_state_in_db(html_balloon_row)
    balloon_check_box_checked = 'checked' if balloon_on else ' '
    balloon_check_box_msg = "Uncheck balloon show. (15% CPU load)" if balloon_on else "Check out a balloon animation."

    speaker_on = config_html.tools_feature_on_off_state_in_db(html_speaker_row)
    speaker_check_box_checked = 'checked' if speaker_on else ' '
    speaker_check_box_msg = "Deselect music animated speaker." if speaker_on else "Check out a music animated speaker."

    style_on = config_html.tools_feature_on_off_state_in_db(html_style_row)
    style_check_box_checked = 'checked' if style_on else ' '
    style_check_box_msg = "Deselect for simplistic style." if style_on else "Check out for brightly colored style."

    blacklist_check_box_off = "9744"
    blacklist_check_box_on = "9989"
    blacklist_is_enabled = mon_rec.status_db_blacklist_get()
    blacklist = mon_rec.blacklist_enabled_button_outfit_get(blacklist_is_enabled)
    blacklist_button_label = ' ON' if blacklist else 'OFF'
    blacklist_btn_msg = "Blacklist feat. is on. Off erases all lists." if blacklist else "Blacklist feat. is off"
    blacklist_check_box_info = blacklist_check_box_on if blacklist else blacklist_check_box_off
    export_path = util_tools.get_export_path()
    return render_template('bp_util_tools.html',
                           export_path=export_path,
                           button_label=blacklist_button_label,
                           button_message=blacklist_btn_msg,
                           enabled=blacklist,
                           blacklist_check_box_info=blacklist_check_box_info,
                           animation_check_box_checked=animation_check_box_checked,
                           animation_check_box_msg=animation_check_box_msg,
                           style_check_box_checked=style_check_box_checked,
                           style_check_box_msg=style_check_box_msg,
                           front_pigs_on=front_pigs_on,
                           front_pigs_check_box_checked=front_pigs_check_box_checked,
                           front_pigs_check_box_msg=front_pigs_check_box_msg,
                           balloon_on=balloon_on,
                           balloon_check_box_checked=balloon_check_box_checked,
                           balloon_check_box_msg=balloon_check_box_msg,
                           speaker_on=speaker_on,
                           speaker_check_box_checked=speaker_check_box_checked,
                           speaker_check_box_msg=speaker_check_box_msg)


@eisenutil_bp.route('/about', methods=('GET', 'POST'))
def about():
    """ render ABOUT page

    POST
       switch browser autostart val (0,1) in db, redirect to refresh the page with updated btn value

    GET
       called from menu, returns var to show download and db path on page

    vars
       is_browser_on - reflect db entry for autostart on btn value
       download_path - parent directory of radio folders
       db_path - location of eisenradio database
    """
    if request.method == 'POST':
        if request.form['browser']:
            lib_eisdb.status_read_status_set(True, 'eisen_intern', 'browser_open', '1')
            return redirect(url_for('eisenutil_bp.about'))

    is_browser_on = lib_eisdb.status_read_status_set(False, 'eisen_intern', 'browser_open', '1')
    download_path = lib_eisdb.get_download_dir()
    db_path = api.config['DATABASE']
    return render_template('bp_util_about.html',
                           is_browser_on=is_browser_on,
                           download_path=download_path,
                           db_path=db_path)


@eisenutil_bp.route('/create', methods=('GET', 'POST'))
def create():
    """"render "New" radio page

    todo eisenApi.radio_name_id_dict[radio_name] = db table id for new radio to assign transparent image
    """
    if request.method == 'POST':
        rv_req = request.form['title']

        title = eis_util.remove_blank(rv_req)
        name_in_db = eis_util.is_in_db_view(title)
        content = request.form['content']
        radio_image = None
        content_type = None

        if not title:
            flash('Name is required!', 'warning')
            return render_template('bp_util_create.html')
        if name_in_db:
            flash('Name is already used!', 'warning')
            return render_template('bp_util_create.html')
        if not content:
            flash('URL is required!', 'warning')
            return render_template('bp_util_create.html')

        # Image
        if request.files['inputFile']:
            file = request.files['inputFile']
            content_type = file.content_type
            print(f' name {file.name} content-type {file.content_type}')

            try:
                db_img = file.read()
                radio_image = lib_eisdb.render_picture(db_img, 'encode')
            except Exception as e:
                print(e)
                radio_image = None
                content_type = None

        else:
            if not name_in_db:
                radio_image, content_type = util_tools.radio_spare_image()

        if request.form['text']:
            text = request.form['text']
        else:
            text = None

        conn = lib_eisdb.get_db_connection()
        posts = conn.execute('SELECT * FROM posts').fetchall()
        # if we later want extra save folders for each radio
        try:
            request_path = posts[0]["download_path"]
        except Exception as e:
            print(e)
            print(' looks like the first radio to create, no save to path set')
            conn.execute('INSERT INTO posts (title, content) VALUES (?, ?)',
                         (title, content))
        else:
            conn.execute('INSERT INTO posts (title, content, download_path, pic_data, pic_content_type, pic_comment) '
                         'VALUES (?, ?, ?, ?, ?, ?)',
                         (title, content, request_path, radio_image, content_type, text))

        conn.commit()
        conn.close()
        return redirect(url_for('eisenhome_bp.index'))

    return render_template('bp_util_create.html')


@eisenutil_bp.route('/save', methods=('GET', 'POST'))
def save():
    """ Render SAVE page, send fail flash message if db empty

    GET push folder name to HTML page
    POST pull new path information from HTML page
    """
    if request.method == 'GET':
        try:
            os.path.abspath(lib_eisdb.get_download_dir())
        except TypeError:
            return render_template('bp_util_save.html', save_to='no-directory-found')
        return render_template('bp_util_save.html', save_to=os.path.abspath(lib_eisdb.get_download_dir()))

    if request.method == 'POST':
        request_path = os.path.abspath(request.form['Path_Save_To'])

        if not request_path:
            flash('A folder is required!', 'warning')
            try:
                os.path.abspath(lib_eisdb.get_download_dir())
            except TypeError:
                return render_template('bp_util_save.html', save_to='no-directory-found')
            return render_template('bp_util_save.html', save_to=os.path.abspath(lib_eisdb.get_download_dir()))

        if request_path:
            made_it = eis_util.make_folder(request_path)
            if not made_it:
                flash('No folder created! exists or denied', 'warning')
                return render_template('bp_util_save.html', save_to=os.path.abspath(lib_eisdb.get_download_dir()))

            conn = lib_eisdb.get_db_connection()
            records = conn.execute('select id from posts').fetchall()
            if not records:
                flash('Nothing in Database!', 'warning')
                try:
                    return render_template('bp_util_save.html', save_to=os.path.abspath(lib_eisdb.get_download_dir()))
                except TypeError:
                    return render_template('bp_util_save.html', save_to='no-row-in-table-create-a-radio')
            for id_num in records:
                # print("Radio Id:", id_num[0])
                conn.execute('UPDATE posts SET download_path = ? WHERE id = ?', (request_path, id_num[0]))
            conn.commit()
            conn.close()

            flash(('Save files to: ' + request_path), "success")
            return redirect(url_for('eisenhome_bp.index'))

        return render_template('bp_util_save.html', save_to=os.path.abspath(lib_eisdb.get_download_dir()))


@eisenutil_bp.route('/<int:id>/edit', methods=('GET', 'POST'))
def edit(id):
    """render Edit page"""
    post = lib_eisdb.get_post(id)

    if request.method == 'POST':
        rv_req = request.form['title']

        title = rv_req.replace(" ", "")
        content = request.form['content']
        print(request.form)
        if not title:
            flash('Title is required!', 'warning')
        # Image, if empty keep db entry as it is
        if request.files['inputFile']:
            file = request.files['inputFile']
            content_type = file.content_type
            print(f' name {file.name} content-type {file.content_type}')
            try:
                db_img = file.read()
                image = lib_eisdb.render_picture(db_img, 'encode')
            except Exception as e:
                print(e)
                image = None
                content_type = None

        else:
            image = None
            content_type = None

        if request.form['text']:
            text = request.form['text']
            if text == 'None':
                text = ' '
        else:
            text = ' '

        conn = lib_eisdb.get_db_connection()
        if image:
            conn.execute('UPDATE posts SET title = ?, content = ?, pic_data = ?, pic_content_type = ?, '
                         'pic_comment = ? WHERE id = ?',
                         (title, content, image, content_type, text, id))
        else:
            conn.execute('UPDATE posts SET title = ?, content = ?, pic_comment = ?  WHERE id = ?',
                         (title, content, text, id))

        conn.commit()
        conn.close()
        return redirect(url_for('eisenhome_bp.index'))

    return render_template('bp_util_edit.html', post=post)


@eisenutil_bp.route('/<int:post_id>')
def post(post_id):
    """draw a page (after clicked on radio picture) to go to radio directory (shout-/icecast) if possible"""
    post = lib_eisdb.get_post(post_id)
    url_port = eis_util.parse_url_simple_url(post["content"])
    pass
    return render_template('bp_util_post.html',
                           post=post,
                           url_port=url_port)


@eisenutil_bp.route('/sw')
def stream_watcher():
    """return the name of a VIP"""
    resp = make_response("stream watcher    (ಠ_ಠ)")
    return resp


@eisenutil_bp.route('/header_info', methods=['POST'])
def header_info():
    """call header_data_read(),return a json list of radio stream header info  [name, bit rate, website, ...]"""
    radio_name = request.form['name']
    rv = request_info.header_data_read(radio_name)
    if not rv:
        rv = '-empty-'
    return jsonify({"header_result": rv})


@eisenutil_bp.route('/delete_info', methods=['GET'])
def delete_info():
    """call inactive_id_read(), return a json list of all inactive radio ids to clean html page

   json_is_data_transfer true if a radio is online (watchdog feeds)
    """
    json_stop_list = stopped_stations.inactive_id_read()
    json_is_data_transfer = eisenApi.radio_active
    return jsonify({"stopped_result": json_stop_list, 'is_data_transfer': json_is_data_transfer})


@eisenutil_bp.route('/listen_info', methods=['GET'])
def listen_info():
    """return complete id of current pressed listen button, js displayLocalPlayListEnableDeactivateListenBtn()

    {listen_btn_id: Listen_17}
    """
    json_listen_btn_id = eis_util.radio_listen_button_get()
    return jsonify({"listen_btn_id": json_listen_btn_id})


@eisenutil_bp.route('/sound/<radio_name>', methods=['GET'])
def sound(radio_name):
    """streams to html audio element

    get_stream_content_type() make an environment variable {radio name: content type} to not read internet header again
    inform browser about mime type, cache control and access control type for connection
    local audio via javascript (use upload functionality)
    add headers
    ['Cache-Control'] = 'no-store'  - forbid browser caching parts of the stream chunks
    ['Access-Control-Allow-Origin'] = '*'   - disable CORS, works only on localhost, not internet
    """
    name = str(radio_name)

    try:
        content_type = os.environ[name]
    except KeyError:
        content_type = browser_stream.get_stream_content_type(name)

    audio_chunks = browser_stream.stream_audio_feed(name)

    response = Response(audio_chunks, mimetype=content_type)
    response.headers['Cache-Control'] = 'no-store'
    response.headers['Access-Control-Allow-Origin'] = '*'
    return response


@eisenutil_bp.route('/blacklist_status_get', methods=['GET'])
def blacklist_status_get():
    """return *enabled* if user has switched on the feature
    """
    html_blacklist_row = 2
    blacklist_enabled = config_html.tools_feature_on_off_state_in_db(html_blacklist_row)
    ret_val = "enabled" if blacklist_enabled else "false"
    return jsonify({"blacklistStatus": ret_val})


@eisenutil_bp.route('/download_dir_get', methods=['GET'])
def download_dir_get():
    """return parent dir of downloads
    """
    try:
        dl_dir = lib_eisdb.get_download_dir()
        print(f"downloads: {dl_dir}")
        return jsonify({"downloadDirGet": dl_dir})
    except Exception as e:
        print("download_dir_get error", e)



"""Tools menu Tools function except Monitor Records (monitor_records.py)

:methods: delete_all_radios(): delete all radios - from db
:methods: export_radios(): export radios - names and urls
:methods: dump_radio_blacklist(): export blacklists
:methods: upload_radios():
:methods: radio_spare_image(): get random image for radio
:methods: upload_blacklists(): blacklist from other app or GhettoRecorder
"""
import configparser
import json
import shutil
import random

import eisenradio.lib.eisdb as eisen_db
import eisenradio.eisenutil.monitor_records as mo_rec

from os import path, remove, listdir
from eisenradio.eisenutil.eisutil import is_in_db_view


def delete_all_radios():
    """delete all radios from db, return True if db is empty"""
    posts = get_radios()
    for radio in posts:
        eisen_db.delete_radio(radio['id'])

    posts = get_radios()
    if not len([radio for radio in posts]):
        return True
    return False


def export_radios():
    """write dump of radio names and urls to ini file and return True

    return False if no radios in db or a physical download directory is non-existing
    make backup of existing ini file
    write download path to [GLOBAL] Section for easy restore
    write radio names and urls to [SECTION] (BLUES_UK = http://149.255.59.3:8232/stream)
    """
    posts = get_radios()
    if not posts:
        return False

    radio_url_dict, export_path = export_radios_dump(posts)
    if not path.exists(export_path):
        return False

    dump_file_path = path.join(export_path, 'settings.ini')
    dump_file_path_backup = path.join(export_path, 'settings.ini_backup')
    dump_radio_export_backup(dump_file_path_backup, dump_file_path)

    print(dump_file_path)
    with open(dump_file_path, 'w') as writer:
        writer.write('[GLOBAL]' + '\n')
        writer.write('SAVE_TO_DIR = ' + export_path + '\n')
        writer.write('[STATIONS]' + '\n')
        for radio, url in radio_url_dict.items():
            writer.write(radio + ' = ' + url + '\n')
        writer.flush()

    return True


def export_radios_dump(posts):
    """return radio_url_dict {radio name: url} and download_path"""
    radio_url_dict = {}
    download_path = ''
    for station in posts:
        station_name = eisen_db.status_read_status_set(False, 'posts', 'title', station['id'])
        station_url = eisen_db.status_read_status_set(False, 'posts', 'content', station['id'])
        download_path = eisen_db.status_read_status_set(False, 'posts', 'download_path', station['id'])
        radio_url_dict[station_name] = station_url
    return radio_url_dict, download_path


def upload_radios(str_ini):
    """call upload_radios_db_import_ini() to upload radios and urls from ini file

    return True, return False if no [STATIONS] section for config parser in file
    """

    config = configparser.ConfigParser()
    config.optionxform = str  # not squeeze the names lower case
    download_path = upload_radios_read_global(config, str_ini)
    name_url_dict = upload_radios_read_stations(config, str_ini)
    if not name_url_dict:
        return False, None
    num_added_db = upload_radios_db_import_ini(name_url_dict, download_path)
    return True, num_added_db


def upload_radios_read_global(config, str_ini):
    """read [GLOBAL] section into a dict, if there is one, to return the download path, return Nothing if none

    some ini don't own a [GLOBAL] section
    GhettoRecorder has no download path in [GLOBAL], ignore error for such files
    """

    try:
        config.read_string(str_ini)
        global_dict = config['GLOBAL']
    except Exception as error:
        print(f'minor error in upload_radios() no [GLOBAL]: {error} - will proceed')
        return
    try:
        download_path = global_dict['SAVE_TO_DIR']
    except Exception as error:
        print(f'minor error in upload_radios() no [SAVE_TO_DIR]: {error} - will proceed')
        # return None, False is bool leads to error later in if
        return
    return download_path


def upload_radios_read_stations(config, str_ini):
    """return config parser [STATIONS] as dict or return Nothing"""
    try:
        config.read_string(str_ini)
        station_dict = config['STATIONS']
    except Exception as error:
        print(f' error in upload_radios(), no mandatory [STATIONS]: {error} give up.')
        # return None, False is bool leads to error in if
        return
    return station_dict


def upload_radios_db_import_ini(station_dict, download_ini_path=None):
    """import names and urls from station_dict into db, function returns nothing

    argument:
       download_ini_path - two scenarios for None
       restore all radios after complete deletion,
       append new radios and no download path available

    Info
       import only unknown radios
       download path is preferred from db, then ini, then write a message as path
    """

    conn = eisen_db.get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    download_db_path = upload_radios_db_get_download_path(posts)

    if download_db_path is not None:
        download_path = download_db_path
    elif download_ini_path is not None:
        download_path = download_ini_path
    else:
        download_path = '/Please/use/Save/from/menu'

    counter = 0
    for radio, url in station_dict.items():
        if is_in_db_view(radio):
            continue
        radio_image, image_content_type = radio_spare_image()  # pull a random image for the new radios
        conn.execute('INSERT INTO posts (title, content, download_path, pic_data, pic_content_type) VALUES ('
                     '?, ?, ?, ?, ?)', (radio, url, download_path, radio_image, image_content_type))
        conn.commit()
        counter += 1
    conn.close()
    return counter


def upload_radios_db_get_download_path(posts):
    """return download path for app, except there is no return Nothing"""
    try:
        download_path = posts[0]["download_path"]
    except Exception as error:
        message = f'no path in db, {error}, proceed.'
        print(message)
        return
    return download_path


def get_radios():
    """return a db dump"""
    conn = eisen_db.get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    conn.close()
    return posts


def radio_spare_image():
    """return a random image with content_type from own package folder"""
    this_dir = path.dirname(__file__)
    # rip off one dir
    app_root = path.dirname(this_dir)
    image_list = listdir(path.join(app_root, 'static', 'images', 'styles', 'small'))
    img_path = path.join(app_root, 'static', 'images', 'styles', 'small', random.choice(image_list))
    with open(img_path, 'rb') as pic_reader:
        image_bin = pic_reader.read()
    img_base64 = eisen_db.render_picture(image_bin, 'encode')
    content_type = 'image/png'
    return img_base64, content_type


def radio_transparent_image():
    """return a transparent image for the radio and content-type for db """
    this_dir = path.dirname(__file__)
    # rip off one dir
    app_root = path.dirname(this_dir)
    img_path = path.join(app_root, 'static', 'images', 'styles', 'small', "radio_transparent.png")

    with open(img_path, 'rb') as pic_reader:
        image_bin = pic_reader.read()
    img_base64 = eisen_db.render_picture(image_bin, 'encode')
    content_type = 'image/png'
    print(f".. radio_transparent_imageradio_transparent_image .{content_type} {img_base64}")
    return img_base64, content_type


def radio_transparent_image_db(radio_name):
    """ return Nothing, replace former radio image with a transparent one """
    conn = eisen_db.get_db_connection()
    radio_image, image_content_type = radio_transparent_image()
    conn.execute('UPDATE posts SET pic_data = ?, pic_content_type = ? WHERE id = ?;',
                 (radio_image, image_content_type, radio_name))
    conn.commit()
    conn.close()


def get_export_path():
    """return the download parent path for radios, or a helper string if no path exists yet"""
    download_path = eisen_db.get_download_dir()
    if not download_path:
        export_path = "use Save from Navigation Menu"
    else:
        export_path = download_path
    return export_path


def dump_radio_blacklist():
    """call dump_write_radio_blacklist() to dump db radio blacklists to one file in json format, return True

    return False if export_path (download_path) not in db
    return False if function call fails, or True if ok
    use parent folder of radios for file storage (json dict)
    create a dump_dict[radio], blacklist dictionary for all radios with a list in value field to export it to a file
    write a message to dump_dict as first entry to clarify file content
    """
    dump_dict = {}
    conn = eisen_db.get_db_connection()
    download_path = conn.execute('SELECT download_path FROM posts;').fetchone()
    export_path = download_path[0]
    radios_list = conn.execute('SELECT title FROM posts').fetchall()

    dump_dict['Eisenradio message'] = json.dumps('This is a json formatted dictionary. Rename radios you do not want.')
    if radios_list:
        for row in radios_list:
            radio_name = row[0]
            file_list = []
            # table column 'display' was used to store current title for displaying, now in api; reuse for list
            known_files = conn.execute('SELECT display FROM posts WHERE title = ?;', (radio_name,)).fetchone()
            if known_files:
                if known_files[0]:
                    try:
                        file_list = json.loads(known_files[0])
                    except Exception as error:
                        print(error)

            if len(file_list) > 0:
                dump_dict[radio_name] = file_list

    conn.close()
    if export_path:
        rv = dump_write_radio_blacklist(dump_dict, export_path)
        return rv
    else:
        return False


def dump_write_radio_blacklist(dump_dict, export_path):
    """dump db list to json file on disk return True if no OS error"""
    dump_file_path = path.join(export_path, 'blacklist.json')
    dump_file_path_backup = path.join(export_path, 'blacklist.json_backup')
    dump_radio_export_backup(dump_file_path_backup, dump_file_path)
    try:
        with open(dump_file_path, 'w') as writer:
            writer.write(json.dumps(dump_dict, indent=4))  # no indent, one long line

        rv = True
    except OSError:
        rv = False
    return rv


def dump_radio_export_backup(dump_file_path_backup, dump_file_path):
    """make a copy from last backup before overwrite file"""
    if path.exists(dump_file_path_backup):
        remove(dump_file_path_backup)
    if path.exists(dump_file_path):
        shutil.copyfile(dump_file_path, dump_file_path_backup)


def upload_blacklists(json_file):
    """return True if dict from json is not empty and no import error

    convert json format to dump_dict
    feed upload_blacklists_import() with list of all radios and dict
    write html if ok or error
    """
    upload_ok = True
    dump_dict = upload_blacklists_dump(json_file)

    if not len(dump_dict) > 0:
        return False, None

    title_count = len([line for f_list in dump_dict.values() if type(f_list) is list for line in f_list])
    radios_list = upload_blacklists_get_radio_names_from_db()
    if radios_list:
        imported_all_ok = upload_blacklists_import(dump_dict, radios_list)
        if not imported_all_ok:
            upload_ok = False
        mo_rec.feed_api_radios_blacklists()
    else:
        print('\n\tImport error. No radios in db.\n')
        upload_ok = False

    if upload_ok:
        print('\n\tBlacklists updated and loaded.\n')
    else:
        print('\n\tSome error occurred. See terminal messages.\n')
    return upload_ok, title_count


def upload_blacklists_import(dump_dict, radios_list):
    """return True if all radios got their lists updated

    loop for radio merge db_list with import_list and write list back to db
    import, merge only if key (RadioName) is matching, upper, lower case
    """
    import_no_err = True
    for row in radios_list:
        radio = row[0]
        if radio in dump_dict.keys():
            print(f'import list for: {radio}')
            print(dump_dict[radio])
            merged_blacklist, rv = upload_blacklists_merge(dump_dict[radio], radio)
            if rv:
                rv = upload_blacklists_write(merged_blacklist, radio)
            if not rv:
                import_no_err = False
    return import_no_err


def upload_blacklists_get_radio_names_from_db():
    """return a list of all radio names in db"""
    conn = eisen_db.get_db_connection()
    radios_list = conn.execute('SELECT title FROM posts').fetchall()
    conn.close()
    return radios_list


def upload_blacklists_dump(json_file):
    """return dict format from json_file"""
    dump_dict = {}
    try:
        dump_dict = json.loads(json_file)
    except Exception as error:
        message = f'unknown error in upload_blacklists_dump() {error}. Give up.'
        print(message)
    return dump_dict


def upload_blacklists_merge(import_list, radio):
    """return concatenated lists and True, except unknown error

    make a set of extended db_list and import_list to remove double values, convert back to list format
    """
    rv = True
    blacklist = mo_rec.dump_radio_blacklist_from_col(radio)
    merged_blacklist = []
    try:
        ext_list = []
        ext_list.extend(blacklist)
        ext_list.extend(import_list)
        merged_blacklist = list(set(ext_list))
    except Exception as error:
        message = f'unknown error in upload_blacklists_merge() {error}, try to proceed'
        print(message)
        rv = False
    return merged_blacklist, rv


def upload_blacklists_write(merged_blacklist, radio):
    """dump lists into db and return True, except unknown error"""
    rv = True
    try:
        conn = eisen_db.get_db_connection()
        conn.execute('UPDATE posts SET display = ? WHERE title = ?;', (json.dumps(merged_blacklist), radio))
        conn.commit()
        conn.close()
    except Exception as error:
        message = f'unknown error in upload_blacklists_write() {error}, try to proceed'
        print(message)
        rv = False
    return rv



from ghettorecorder import ghettoApi


class Api:
    """class in a 3rd party module mirrors the flask variables

    to get the variables without the (with app.app_context:) statement of flask used
    experimenting with android, which do not like(d) application factory and app_context
    default class constructor without arguments
    init vars:
    config -- flask config (default None)
    """

    def __init__(self):
        """default without args"""

        self.config = None

    def init_app(self, app):
        """flask config
        compare with os vars to see if start config db path got an accident
        trouble with testing from partial initialized functional test
        """

        self.config = app.config

    def __repr__(self):
        return f"api.init_app({self.config})"

    def __str__(self):
        return f"Flask application factory config: {self.config}"


class EisenTest:
    def __init__(self):
        self.thread_killer = ()

    def init_test_thread_killer(self, thread_killer):
        """kills a single thread ("radio", "listen", "meta"), thread reads tuple and exit if match"""
        self.thread_killer = thread_killer


class EisenApi:
    """ EisenRadio is a frontend and runs on top of GhettoRecorder
    attributes needed only for eisenradio modules, some api modules connect to GhettoApi

    """

    def __init__(self):
        """ init

        Attributes:
           radio_name_id_dict . {id: name} {starRadio: 1, planetRadio: 2} radio name to db table id mapping
           radio_id_name_dict . {name: id} radios listed on startpage, key:db id, val: radio name shown on html
           skipped_record_eisen_dict . {radio1: 2} compare list len of Ghetto skipped_in_session_dict with counter
           call_skipped_record_eisen_dict . endpoint call before get_skipped_record()
           lis_btn_dict . listen, radio get on/off entry for radio table id
           rec_btn_dict . record, radio get on/off entry for radio table id
           show_analyser . cookie for analyser, can have db entry but want cookie to show howto
           work_port . wsgi port number, for functions in java to connect to correct endpoint
           radio_active . shows if a thread is connected to the internet, green light in console
        """
        self.radio_name_id_dict = {}
        self.radio_id_name_dict = {}
        self.call_radio_name_id_dict = True
        self.skipped_record_eisen_dict = {}
        self.call_skipped_record_eisen_dict = True
        self.lis_btn_dict = {}
        self.rec_btn_dict = {}
        self.show_analyser = True
        self.work_port = None
        self.radio_active = False

    def init_radio_name_id_dict(self):
        """ key is radio_id: val is name radio_name_id_dict {table_id: name},
        radio_id_name_dict {name: table_id}
        todo change names to better reflect purpose, radio_name_id_dict, radio_id_name_dict
        """
        if self.call_radio_name_id_dict:
            self.radio_name_id_dict = {name: table_id for table_id, name in self.radio_id_name_dict.items()}
            self.call_radio_name_id_dict = False

    def init_radio_id_name_dict(self, radio_id_name_dict):
        """ key:db id, val: radio name shown on html """
        self.radio_id_name_dict = radio_id_name_dict

    def init_skipped_record_eisen_dict(self):
        """ get radio key name from GhettoApi all_blacklists_dict {radio1: [], radio2: []} ([] is title skip list)
        set key value to zero to see if GhettoApi skipped_in_session_dict was updated later
        only called once
        """
        if self.call_skipped_record_eisen_dict:
            self.skipped_record_eisen_dict = {radio: 0 for radio in ghettoApi.all_blacklists_dict.keys()}
            self.call_skipped_record_eisen_dict = False

    def get_skipped_record(self, radio_name):
        """ return True if a title was skipped since last request, False if not
        check EisenApi skipped_record_eisen_dict radio:count lower than in length of GhettoApi skipped_in_session_dict
        update skipped_record_eisen_dict[radio_name] to new length of ghettoApi.skipped_in_session_dict[radio_name] list
        """
        if not len(ghettoApi.skipped_in_session_dict):
            return False
        if radio_name not in ghettoApi.skipped_in_session_dict.keys():
            return False

        if self.skipped_record_eisen_dict[radio_name] < len(ghettoApi.skipped_in_session_dict[radio_name]):
            self.skipped_record_eisen_dict[radio_name] = len(ghettoApi.skipped_in_session_dict[radio_name])
            return True
        else:
            return False

    def init_lis_btn_dict(self, lis_btn_dict):
        """ key:db id, val: 0 or 1, button down 1 """
        self.lis_btn_dict = lis_btn_dict

    def init_rec_btn_dict(self, rec_btn_dict):
        """ key:db id, val: 0 or 1, button down 1 """
        self.rec_btn_dict = rec_btn_dict

    def init_show_analyser(self, show_analyser):
        """ set cookie to show or hide spectrum analyser, true or false """
        self.show_analyser = show_analyser

    def init_work_port(self, work_port):
        """ port number on startup in wsgi.py or app.py: write the sound endpoint url to java """
        self.work_port = work_port

    def init_radio_active(self, radio_active):
        """ return True if a thread with action string record or listen is connected to the internet """
        self.radio_active = radio_active


api = Api()
eisenTest = EisenTest()
eisenApi = EisenApi()



from os import environ
from time import sleep
from ghettorecorder import ghettoApi


def stream_audio_feed(name):
    """yield buffer, drain queue (cleanup) in ghetto_audio_stream_dict[name + ',audio'] if listen btn gets inactive
    Raise:
        HTML audio element connects earlier than buffer is prepared
    """
    while True:

        try:
            if not ghettoApi.listen_active_dict[name]:
                if not ghettoApi.ghetto_audio_stream_dict[name + ',audio'].empty():
                    ghettoApi.ghetto_audio_stream_dict[name + ',audio'].get()
                return

            if not ghettoApi.ghetto_audio_stream_dict[name + ',audio'].empty():
                yield ghettoApi.ghetto_audio_stream_dict[name + ',audio'].get()

        except KeyError:
            pass


def get_stream_content_type(name):
    """loop until return radios content type, create var {radio: content type}, route don't call def again"""
    content_type = ''
    while not content_type:

        try:
            content_type = ghettoApi.ghetto_measure_dict[name + ',suffix']
            environ[name] = content_type
        except KeyError:
            pass

        sleep(.1)
    return content_type



import os
import threading
import time
from flask import Blueprint, render_template, request, flash, redirect, url_for, make_response, jsonify
from eisenradio.eisenutil import config_html
import eisenradio.eisenhome.eishome as eis_home
from eisenradio.lib.eisdb import get_post, delete_radio, enum_radios, get_db_connection
from ghettorecorder import ghettoApi
from eisenradio.api import eisenApi
import ghettorecorder.ghetto_recorder as ghetto

# Blueprint Configuration
eisenhome_bp = Blueprint(
    'eisenhome_bp', __name__,
    template_folder='bp_home_templates',
    static_folder='bp_home_static',
    static_url_path='/bp_home_static'
)


@eisenhome_bp.route('/', methods=('GET', 'POST'))
def index():
    """draws start page

    dump db and run eis_home.index_first_run() to prepare app, fill dicts, start daemons
    collect vars for jinja to draw start page from template

    POST: js sends post with button click event listener, / index endpoint return json dict to js
    return: vars for looping over the html elements and provide names of radios, button color, title for console

    * posts - db dump
    * combo_master_timer - actual timer value call eis_home.combo_master_timer (default 0)
    * status_listen_btn_dict - whole dict eis_home.status_listen_btn_dict to set button colors if pressed (default 0)
    * status_record_btn_dict - whole dict eis_home.status_record_btn_dict to set button colors if pressed (default 0)
    * current_station - write name of listened station to console if any (default None)
    * current_table_id - table id of listened station (default None)
    * listen_last_url - feed the audio element with url after page refresh (default None)
    """
    local_host_sound_route = "http://localhost:" + str(eisenApi.work_port) + "/sound/"
    listen_last_url = ""

    posts = enum_radios()
    eis_home.index_first_run(posts)
    # first call returns None
    current_station, current_table_id = eis_home.curr_radio_listen()

    if request.method == 'POST':
        # print_request_values(request.form.values())
        post_request = request.form.to_dict()  # flat dict werkzeug doc
        # $("button").click(function () {}, ajax /  TODO ret must be split into multiple fun calls
        json_post = eis_home.index_posts_clicked(post_request)
        # returns False on error to deactivate buttons for newly created radios, app restart required
        if json_post:
            return json_post

    # for page reload, connect to last sound endpoint again
    if current_station:
        listen_last_url = local_host_sound_route + current_station

    [print(row[0], row[2]) for row in posts if row["id"]]

    return render_template('bp_home_index.html',
                           posts=posts,
                           combo_master_timer=eis_home.combo_master_timer,
                           status_listen_btn_dict=eis_home.status_listen_btn_dict,
                           status_record_btn_dict=eis_home.status_record_btn_dict,
                           current_station=current_station,
                           current_table_id=current_table_id,
                           listen_last_url=listen_last_url)


@eisenhome_bp.route('/enable_sound_endpoint', methods=['POST'])
def enable_sound_endpoint():
    """Get radio id.
    Disable/Enable endpoint and return URL with port num for JS audio element.
    """
    table_id = request.form['radioNum']
    radio_name = request.form['radioName']
    end_points = [radio for radio in ghettoApi.listen_active_dict.keys()]
    for radio in end_points:
        ghettoApi.listen_active_dict[radio] = False  # all sound endpoints stop yielding audio data, pretty or not
    time.sleep(1)
    [print(thread.name) for thread in threading.enumerate()]

    eis_home.status_listen_btn_dict[table_id] = 1  # register listen
    eis_home.dispatch_master(int(table_id), 'Listen', 1)  # enable via "normal" key press ON
    endpoint = "http://localhost:" + str(eisenApi.work_port) + "/sound/" + radio_name
    return jsonify({"newAudioSource": endpoint})


@eisenhome_bp.route('/enable_recorder', methods=['POST'])
def enable_recorder():
    """Big mess so far. Must go GhettoRecorder 3 to get rid of the buttons.
    V3 simply start, stop feeding a queue itself and is easy to kill.

    Problem (have solution)
    App had a record and a listen button. Dictionaries keep track of the up/down status of the buttons.
    dispatch_record_btn() writes multiple dicts and starts recorder threads.
    ghettoApi.listen_active_dict is there, but I forgot a ghettoApi.record_active_dict! :)

    eis_home.status_listen_btn_dict = {}  # {15: 1 } {"table_id 15": "pressed down"}
    eis_home.status_record_btn_dict = {} PLUS eis_home.active_streamer_dict[radio_name] = str(table_id)

    Solution for this version
    eis_home.status_listen_btn_dict; must be cleaned and written in enable_sound_endpoint()
    eis_home.status_record_btn_dict; enable_recorder()
    eis_home.active_streamer_dic; enable_recorder()

    Get radio id.
    Run recorder if it is not already running.
    """
    table_id = request.form['radioNum']
    radio_name = request.form['radioName']
    json_str = "No Data Yet, use pull interval eis_home.active_streamer_dict"
    if radio_name in eis_home.active_streamer_dict.keys():
        json_str = "recorder runs already " + radio_name
        return jsonify({"activeRecorderNameId": json_str})

    eis_home.active_streamer_dict[radio_name] = str(table_id)  # register name id
    eis_home.status_record_btn_dict[table_id] = 1  # record button pressed
    eis_home.dispatch_master(int(table_id), 'Record', 1)
    time.sleep(1)
    [print(thread.name) for thread in threading.enumerate()]
    return jsonify({"activeRecorderNameId": json_str})


@eisenhome_bp.route('/disable_recorder', methods=['POST'])
def disable_recorder():
    """ Revert settings of enable_recorder().

    Make sure the recorder thread is offline. radio_pan_america PB all others ok
    """
    table_id = request.form['id']
    radio_name = request.form['name']
    ghetto.GRecorder.record_active_dict[radio_name] = False
    if radio_name in eis_home.active_streamer_dict.keys():
        del eis_home.active_streamer_dict[radio_name]
    eis_home.status_record_btn_dict[table_id] = 0
    return jsonify({"disabledRecorder": radio_name + " " + table_id})  # need a return action


@eisenhome_bp.route('/degrade_animation_level_get', methods=['GET'])
def degrade_animation_level_get():
    """ return the animation level status for CPU utilisation from db, high is 1, low is 0,
    also called on js onLoad to fill the animationsAllowedDict
    db return value is a sql row object, one must choose the column
    jsonify first key is name of the caller function in js to separate (follow) ajax return values better
    """
    row_html_cpu = 8
    conn = get_db_connection()
    on = conn.execute('SELECT browser_open FROM eisen_intern WHERE id =' + str(row_html_cpu) + ';').fetchone()
    conn.close()
    cpu_utilization = "high" if on[0] else "low"
    return jsonify({"degradeAnimationsWriteDict": cpu_utilization})


@eisenhome_bp.route('/degrade_animation_level_set', methods=['GET'])
def degrade_animation_level_set():
    """ degrade_animation_level_set() and degrade_animation_level_get() called on radio button change to
    write the js animationsAllowedDict new, so animation can see if run is allowed

    | write the animation level status for CPU to db, high is 1 (default in db), low is 0
    | We can only use 1 and 0 values since db writer uses on/off style, default is 1.

    | config_html.tools_feature_settings_get_rows() compares actual rows number in db with rows hardcoded as a list
    | it calls tools_feature_settings_create_default(feature_without_row) to create the rows with default value
    | config_html.tools_feature_settings_get_rows is called on startup to write new rows if version upgrade of package
    | database row 8 for html_cpu
    """
    on = 1
    off = 0
    row_html_cpu = 8
    conn = get_db_connection()
    status = conn.execute('SELECT browser_open FROM eisen_intern WHERE id =' + str(row_html_cpu) + ';').fetchone()
    conn.close()
    toggle = off if status[0] else on
    config_html.tools_feature_toggle_show_html_config(toggle, row_html_cpu)
    return jsonify({"degradeAnimationsSet": "required response"})


@eisenhome_bp.route('/tools_radio_config_get', methods=['GET'])
def tools_radio_config_get():
    """ return a dictionary with HTML settings, tell the js how to style the app: style, animations;

    config_html.tools_feature_settings_get_rows() has all rows hardcoded as a list
    """
    row_db_html_animation = 3
    row_db_html_style = 4
    row_db_html_front_pigs = 5
    row_db_html_balloon = 6
    row_db_html_speaker = 7
    row_db_html_cpu = 8

    row_list = [
        row_db_html_animation,
        row_db_html_style,
        row_db_html_front_pigs,
        row_db_html_balloon,
        row_db_html_speaker,
        row_db_html_cpu,
    ]
    feature_list = [
        "checkboxConfigAnimation",
        "checkboxConfigStyle",
        "checkboxConfigFrontPigs",
        "checkboxConfigBalloon",
        "checkboxConfigFlatSpeaker",
        "cpuUtilisation",
    ]
    check_box_status_list = list(map(lambda row: config_html.tools_feature_on_off_state_in_db(row), row_list))
    feature_checked_tuple = zip(feature_list, check_box_status_list)
    settings_dict = {feature: checked for feature, checked in feature_checked_tuple}

    return jsonify({"configEisenradioHtmlSetting": settings_dict})


@eisenhome_bp.route('/page_flash', methods=('GET', 'POST'))
def page_flash():
    """reset timer vars, return flash info on info page"""
    eis_home.combo_master_timer = 0  # master timer recording
    eis_home.progress_master_percent = 0

    flash('Count down timer ended all activities. App restart recommended!', 'success')
    return render_template('bp_home_page_flash.html')


@eisenhome_bp.route('/<int:id>/delete', methods=['POST'])
def delete(id):
    """delete a radio only if it is not active, return a flash of ok or not, redirect to start page"""
    if eis_home.status_listen_btn_dict[id] or eis_home.status_record_btn_dict[id]:
        flash('Radio is active. No deletion.', 'warning')
        return redirect(url_for('eisenhome_bp.index'))

    post = get_post(id)
    rv = delete_radio(id)
    if rv:
        flash('"{}" was successfully deleted!'.format(post['title']), 'success')
    if not rv:
        flash('"{}" was not deleted!'.format(post['title']), 'warning')
    return redirect(url_for('eisenhome_bp.index'))


@eisenhome_bp.route('/cookie_set_dark', methods=['GET'])
def cookie_set_dark():
    """darkmode cookie

    secure=False for http and https
    localhost is an inner Network of the computer used by Eisenradio server and browser, therefore a save place
    secure=True without external signed ssl certs will NOT rise security nor let disappear dev console errors in IEx
    """
    resp = make_response("Eisenkekse sind die besten")
    resp.set_cookie(
        'eisen-cookie',
        'darkmode',
        max_age=60 * 60 * 24 * 365 * 2,
        httponly=True,
        secure=False,
        samesite='Strict'
    )
    return resp


@eisenhome_bp.route('/cookie_get_dark', methods=['GET'])
def cookie_get_dark():
    """Return dark mode set or not."""
    mode = request.cookies.get('eisen-cookie', None)
    return jsonify({"darkmode": mode})


@eisenhome_bp.route('/cookie_del_dark', methods=['GET'])
def cookie_del_dark():
    resp = make_response("necesito nuevas cookies")
    resp.set_cookie('eisen-cookie', max_age=0, samesite='Strict')
    return resp


@eisenhome_bp.route('/streamer_get', methods=['GET'])
def streamer_get():
    """return json dict of all active streams rec;

    populate dropdown in console after page refresh
    empty dict if no streamer, else send name and db table id
    """
    streamer_dict = {}
    if len(eis_home.active_streamer_dict) > 0:
        for rec_station, rec_station_id in eis_home.active_streamer_dict.items():
            streamer_dict[rec_station] = rec_station_id
    return jsonify({'streamerGet': streamer_dict})


@eisenhome_bp.route('/index_posts_combo', methods=['POST'])
def index_posts_combo():
    """set hours for timer, return read val for test"""
    eis_home.combo_master_timer = request.form['timeRecordSelectAll']
    return eis_home.combo_master_timer


@eisenhome_bp.route('/index_posts_percent', methods=['GET'])
def index_posts_percent():
    """return percent of timer to js"""
    return jsonify({'result': eis_home.progress_master_percent, "timer": eis_home.combo_master_timer})


@eisenhome_bp.route('/svg_symbol_get', methods=['GET'])
def svg_symbol_get():
    """
    """
    svg_symbol = "svgImagesSymbol.svg"
    this_module_dir = os.path.dirname(__file__)
    symbol_folder = os.path.join(this_module_dir, "bp_home_static", "images", svg_symbol)
    with open(symbol_folder, "rb") as reader:
        svg_content = reader.read()
    return svg_content



"""FLASK server on port 5050.
Get a good observation of html connections.
WAITRESS wsgi server can run on a random port - wsgi.py.
"""
import sys
import socket
import eisenradio
from os import path

# flask wants its config
instance_dir = path.abspath(path.join(path.dirname(__file__), 'instance'))
sys.path.append(path.abspath(instance_dir))
port = 5050


def is_port_occupied(srv_port: int) -> bool:
    """The error indicator is 0 if the operation succeeded, otherwise return the value of the errno variable.
    """
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(('localhost', srv_port)) == 0


def main():
    global port
    while 1:
        if not is_port_occupied(port):
            app = eisenradio.create_app(port)  # transfer port num info to flask (audio endpoint ...:5050/sound/starFM)
            app.run('localhost', port)
            break
        else:
            print(f'Port in use: {port} \n\tadd one to port number')
            port += 1


if __name__ == "__main__":
    main()



import sqlite3
import base64

from os import path, makedirs, remove
from pathlib import Path as Pathlib_path
from flask import flash
from werkzeug.exceptions import abort
from eisenradio.db import make_db_from_schema
from eisenradio.api import api


def get_db_path():
    return api.config['DATABASE']


def get_db_connection():
    conn = sqlite3.connect(api.config['DATABASE'])
    conn.row_factory = sqlite3.Row
    return conn


def status_read_status_set(set_status, table, column, table_id):
    """return value of db or set a value in db

    helper function for often used queries
    can only set values in eisen_intern table
    """
    col_value = ''

    conn = get_db_connection()

    if table == 'posts':
        """ tuple one field set(id,) """
        if column == 'content':
            col_value = conn.execute('SELECT content FROM posts WHERE id = ? ;', (table_id,)).fetchone()
        if column == 'title':
            col_value = conn.execute('SELECT title FROM posts WHERE id = ? ;', (table_id,)).fetchone()
        if column == 'download_path':
            col_value = conn.execute('SELECT download_path FROM posts WHERE id = ? ;', (table_id,)).fetchone()
        conn.close()
        if col_value:
            return col_value[0]
        if not col_value:
            return "column not in table posts, status_read_status_set"

    if table == 'eisen_intern':
        """ tuple two fields are set (id,str) """
        if column == 'browser_open':
            col_value = conn.execute(
                'SELECT browser_open FROM eisen_intern WHERE id = ?;', (str(table_id))).fetchone()

            if set_status:
                if col_value[0]:  # toggle values
                    conn.execute('UPDATE eisen_intern SET browser_open = ? WHERE id = ?;', (str(0), str(table_id)))
                else:
                    conn.execute('UPDATE eisen_intern SET browser_open = ? WHERE id = ?;', (str(1), str(table_id)))

        conn.commit()
        conn.close()

    return col_value[0]


def get_download_dir():
    """return download dir, return None if db is empty"""
    conn = get_db_connection()
    column = conn.execute('SELECT download_path FROM posts').fetchall()
    conn.close()
    if column:
        if column[0]:
            for row in column:
                return row[0]
    message = ' No Radio in Database! Create a radio and set Save path in navigation bar'
    print(message)
    flash(message, 'warning')
    return


def get_db_smaller():
    db = api.config['DATABASE']
    conn = sqlite3.connect(db, isolation_level=None)
    conn.execute("VACUUM")
    conn.close()


def render_picture(byte_data, de_enc):
    render_pic = ''
    if de_enc == 'encode':
        render_pic = base64.b64encode(byte_data).decode('ascii')
    if de_enc == 'decode':
        render_pic = base64.b64decode(byte_data).decode('ascii')
    return render_pic


def get_post(post_id):
    conn = get_db_connection()
    post = conn.execute('SELECT * FROM posts WHERE id = ?',
                        [str(post_id)]).fetchone()
    conn.close()
    if post is None:
        abort(404)  # flask.abort(404)
    return post


def install_new_db(db_path):
    """make db from schema"""
    if not path.exists(db_path):
        try:
            makedirs(path.dirname(db_path))
        except FileExistsError:
            pass

    if not Pathlib_path(db_path).is_file():
        make_db_from_schema(Pathlib_path(db_path))


def delete_db(db_path):
    if Pathlib_path(db_path).is_file():
        remove(Pathlib_path(db_path))


def delete_radio(radio_id):
    conn = get_db_connection()
    conn.execute('DELETE FROM posts WHERE id = ?', (str(radio_id),))
    conn.commit()
    conn.close()
    get_db_smaller()
    return True


def enum_radios():
    conn = get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    conn.close()
    return posts


def db_get_id_from_name(radio_name):
    conn = get_db_connection()
    radio_id = conn.execute('SELECT id FROM posts WHERE title = ?', (str(radio_name),)).fetchone()
    conn.close()
    return radio_id[0]
